@@ -142,7 +142,18 @@ oper.e.generic_spct <- function(e1, e2, oper) {
         setSourceSpct(z, time.unit=getTimeUnit(e1), bswf.used = getBSWFUsed(e1),
                       strict.range = getOption("photobiology.strict.range",
                                                default = FALSE))
-      } else if (filter.quantity=="absorbance") {
+      } else if (filter.quantity=="absorptance") {
+        if (!identical(oper, `*`) && !identical(oper, `/`)) {
+          warning("Only '*' and '/' are allowed between source_spct and filter_spct objects")
+          return(NA)
+        }
+        T2Afr(e2, action = "add", byref = TRUE)
+        z <- oper_spectra(e1$w.length, e2$w.length,
+                          e1$s.e.irrad, e2$Afr,
+                          bin.oper=oper, trim="intersection")
+        names(z)[2] <- "s.e.response"
+        setResponseSpct(z, time.unit=getTimeUnit(e1))
+      }  else if (filter.quantity=="absorbance") {
         if (!identical(oper, `*`) && !identical(oper, `/`)) {
           warning("Only '*' and '/' are allowed between source_spct and filter_spct objects")
           return(NA)
@@ -248,6 +259,44 @@ oper.e.generic_spct <- function(e1, e2, oper) {
         warning("The operation attempted is undefined according to Optics laws or the input is malformed")
         return(NA)
       }
+    } else if (filter.quantity=="absorptance") {
+      T2Afr(e1, action = "add", byref = TRUE)
+      if (is.numeric(e2)) {
+        z <- e1
+        if (exists("Tfr", z, inherits = FALSE)) {
+          z[["Tfr"]] <- NULL
+        }
+        z[["Afr"]] <- oper(z[["Afr"]], e2)
+        check_spct(z)
+        return(z)
+      } else if (class2 == "source_spct") {
+        q2e(e2, action = "add", byref = TRUE)
+        if (!identical(oper, `*`) && !identical(oper, `/`)) {
+          warning("Only '*' and '/' are allowed between filter_spct and source_spct objects")
+        }
+        z <- oper_spectra(e1$w.length, e2$w.length,
+                          e1$Afr, e2$s.e.irrad,
+                          bin.oper=oper, trim="intersection")
+        names(z)[2] <- "s.e.response"
+        setResponseSpct(z, time.unit=getTimeUnit(e2))
+        return(z)
+      } else if (class2 == "filter_spct") {
+        T2Afr(e2, action = "add", byref = TRUE)
+        if (!identical(oper, `*`) && !identical(oper, `/`)) {
+          warning("Only '*' and '/' are allowed for two filter_spct objects (absorptance)")
+          return(NA)
+        }
+        z <- oper_spectra(e1$w.length, e2$w.length,
+                          e1$Afr, e2$Afr,
+                          bin.oper=oper, trim="intersection")
+        names(z)[2] <- "Afr"
+        setFilterSpct(z, strict.range = getOption("photobiology.strict.range",
+                                                  default = FALSE))
+        return(z)
+      } else { # this traps optically illegal operations
+        warning("The operation attempted is undefined according to Optics laws or the input is malformed")
+        return(NA)
+      }
     } else if (filter.quantity=="absorbance") {
       T2A(e1, action = "add", byref = TRUE)
       if (is.numeric(e2)) {
@@ -358,12 +407,12 @@ oper.e.generic_spct <- function(e1, e2, oper) {
                            x = oper(e1$x, e2["x"]),
                            y = oper(e1$y, e2["y"]),
                            z = oper(e1$z, e2["z"])))
-        } else {
-          return(chroma_spct(w.length = e1$w.length,
-                             x = oper(e1$x, e2),
-                             y = oper(e1$y, e2),
-                             z = oper(e1$z, e2)))
-         }
+      } else {
+        return(chroma_spct(w.length = e1$w.length,
+                           x = oper(e1$x, e2),
+                           y = oper(e1$y, e2),
+                           z = oper(e1$z, e2)))
+      }
     } else if (class2 == "chroma_spct") {
       x <- oper_spectra(e1$w.length, e2$w.length,
                         e1$x, e2$x,
@@ -412,6 +461,11 @@ oper.e.generic_spct <- function(e1, e2, oper) {
         return(filter_spct(w.length=e2$w.length, Tfr=oper(e1, e2$Tfr),
                            strict.range = getOption("photobiology.strict.range",
                                                     default = FALSE)))
+      } else if (filter.quantity=="absorptance") {
+        T2Afr(e2, action = "add", byref = TRUE)
+        return(filter_spct(w.length=e2$w.length, Afr = oper(e1, e2$Afr),
+                           strict.range = getOption("photobiology.strict.range",
+                                                    default = FALSE)))
       } else if (filter.quantity=="absorbance") {
         T2A(e2, action = "add", byref = TRUE)
         return(filter_spct(w.length=e2$w.length, A=oper(e1, e2$A),
@@ -592,6 +646,14 @@ oper.q.generic_spct <- function(e1, e2, oper) {
         setSourceSpct(z, time.unit = getTimeUnit(e1), bswf.used = getBSWFUsed(e1),
                       strict.range = getOption("photobiology.strict.range",
                                                default = FALSE))
+      } else if (filter.quantity=="absorptance") {
+        if (!identical(oper, `*`)) return(NA)
+        T2Afr(e2, action = "add", byref = TRUE)
+        z <- oper_spectra(e1$w.length, e2$w.length,
+                          e1$s.q.irrad, e2$Afr,
+                          bin.oper=oper, trim="intersection")
+        names(z)[2] <- "s.q.response"
+        setResponseSpct(z)
       } else if (filter.quantity=="absorbance") {
         if (!identical(oper, `*`)) return(NA)
         T2A(e2, action = "add", byref = TRUE)
@@ -674,6 +736,37 @@ oper.q.generic_spct <- function(e1, e2, oper) {
         warning("The operation attempted is undefined according to Optics laws or the input is malformed")
         return(NA)
       }
+    } else if (filter.quantity=="absorptance") {
+      T2Afr(e1, action = "add", byref = TRUE)
+      if (is.numeric(e2)) {
+        if (exists("Tfr", z, inherits = FALSE)) {
+          z[["Tfr"]] <- NULL
+        }
+        z[["Afr"]] <- oper(z[["Afr"]], e2)
+        return(z)
+      } else if (class2 == "source_spct") {
+        e2q(e2, action = "add", byref = TRUE)
+        if (!identical(oper, `*`)) return(NA)
+        z <- oper_spectra(e1$w.length, e2$w.length,
+                          e1$Afr, e2$s.q.irrad,
+                          bin.oper=oper, trim="intersection")
+        names(z)[2] <- "s.q.response"
+        setResponseSpct(z)
+        return(z)
+      } else if (class2 == "filter_spct") {
+        T2A(e2, action = "add", byref = TRUE)
+        if (!identical(oper, `+`) || !identical(oper, `-`)) return(NA)
+        z <- oper_spectra(e1$w.length, e2$w.length,
+                          e1$A, e2$A,
+                          bin.oper=oper, trim="intersection")
+        names(z)[2] <- "A"
+        setFilterSpct(z, strict.range = getOption("photobiology.strict.range",
+                                                  default = FALSE))
+        return(z)
+      } else { # this traps optically illegal operations
+        warning("The operation attempted is undefined according to Optics laws or the input is malformed")
+        return(NA)
+      }
     } else if (filter.quantity=="absorbance") {
       T2A(e1, action = "add", byref = TRUE)
       if (is.numeric(e2)) {
@@ -804,6 +897,9 @@ oper.q.generic_spct <- function(e1, e2, oper) {
       if (filter.quantity=="transmittance") {
         A2T(e2, action = "add", byref = TRUE)
         return(filter_spct(w.length=e2$w.length, Tfr=oper(e1, e2$Tfr), strict.range = getOption("photobiology.strict.range", default = FALSE)))
+      } else if (filter.quantity=="absorptance") {
+        T2Afr(e2, action = "add", byref = TRUE)
+        return(filter_spct(w.length=e2$w.length, Afr=oper(e1, e2$Afr), strict.range = getOption("photobiology.strict.range", default = FALSE)))
       } else if (filter.quantity=="absorbance") {
         T2A(e2, action = "add", byref = TRUE)
         return(filter_spct(w.length=e2$w.length, A=oper(e1, e2$A), strict.range = getOption("photobiology.strict.range", default = FALSE)))
@@ -1323,7 +1419,7 @@ tanpi.generic_spct <- function(x) {
 
 #' Convert absorbance into transmittance
 #'
-#' Function that coverts absorbance into transmittance (fraction).
+#' Function that coverts absorbance (a.u.) into transmittance (fraction).
 #'
 #' @param x an R object
 #' @param action a character string
@@ -1382,27 +1478,31 @@ A2T.filter_mspct <- function(x, action = "add", byref = FALSE, ...) {
 
 #' Convert transmittance into absorbance.
 #'
-#' Function that coverts transmittance into absorbance (fraction).
+#' Function that coverts transmittance (fraction) into absorbance (a.u.).
 #'
 #' @param x an R object
 #' @param action character Allowed values "replace" and "add"
 #' @param byref logical indicating if new object will be created by reference or by copy of x
+#' @param clean logical replace off-boundary values before conversion
 #' @param ... not used in current version
 #'
 #' @export T2A
 #' @family quantity conversion functions
 #'
-T2A <- function(x, action, byref, ...) UseMethod("T2A")
+T2A <- function(x, action, byref, clean, ...) UseMethod("T2A")
 
 #' @describeIn T2A Default method for generic function
 #'
 #' @export
 #'
-T2A.default <- function(x, action=NULL, byref = FALSE, ...) {
-  if (any(x < 0)) {
-    Tfr.zero <- getOption("photobiology.Tfr.zero", default = 1e-10)
-    warning("Replacing zeros by", Tfr.zero)
-    x <- ifelse(x <= 0, Tfr.zero, x)
+T2A.default <- function(x, action=NULL, byref = FALSE, clean = TRUE, ...) {
+  if (clean) {
+    Tfr.zero <- getOption("photobiology.Tfr.zero", default = 0)
+    if (any(x < Tfr.zero)) {
+      warning("Replacing ", sum(x < Tfr.zero), " values < ",
+              Tfr.zero, " with ", Tfr.zero)
+      x <- ifelse(x <= 0, Tfr.zero, x)
+    }
   }
   return(-log10(x))
 }
@@ -1411,13 +1511,17 @@ T2A.default <- function(x, action=NULL, byref = FALSE, ...) {
 #'
 #' @export
 #'
-T2A.filter_spct <- function(x, action="add", byref = FALSE, ...) {
+T2A.filter_spct <- function(x, action="add", byref = FALSE, clean = TRUE, ...) {
   if (byref) {
     name <- substitute(x)
   }
   if (exists("A", x, inherits=FALSE)) {
     NULL
   } else if (exists("Tfr", x, inherits=FALSE)) {
+    if (clean) {
+      # we need to avoid infinite recursion
+      using_Tfr(x <- clean(x))
+    }
     x[["A"]] <- -log10(x[["Tfr"]])
   } else {
     x[["A"]] <- NA
@@ -1439,10 +1543,255 @@ T2A.filter_spct <- function(x, action="add", byref = FALSE, ...) {
 #'
 #' @export
 #'
-T2A.filter_mspct <- function(x, action = "add", byref = FALSE, ...) {
-  msmsply(x, T2A, action = action, byref = byref, ...)
+T2A.filter_mspct <- function(x, action = "add", byref = FALSE, clean = TRUE, ...) {
+  msmsply(x, T2A, action = action, byref = byref, clean = clean, ...)
 }
 
+# T2Afr ---------------------------------------------------------------------
+
+#' Convert transmittance into absorptance.
+#'
+#' Function that coverts transmittance (fraction) into absorptance (fraction).
+#' If reflectance (fraction) is available, it allows conversions between
+#' internal and total absorptance.
+#'
+#' @param x an R object
+#' @param action character Allowed values "replace" and "add"
+#' @param byref logical indicating if new object will be created by reference or by copy of x
+#' @param clean logical replace off-boundary values before conversion
+#' @param ... not used in current version
+#'
+#' @export T2Afr
+#' @family quantity conversion functions
+#'
+#' @examples
+#' T2Afr(Ler_leaf.spct)
+#'
+T2Afr <- function(x, action, byref, clean, ...) UseMethod("T2Afr")
+
+#' @describeIn T2Afr Default method for generic function
+#'
+#' @export
+#'
+T2Afr.default <- function(x, action=NULL, byref = FALSE, clean = FALSE, ...) {
+  if (clean) {
+    Tfr.zero <- getOption("photobiology.Tfr.zero", default = 0)
+    if (any(x < Tfr.zero)) {
+      warning("Replacing ", sum(x < Tfr.zero), " values < ",
+              Tfr.zero, " with ", Tfr.zero)
+      x <- ifelse(x <= 0, Tfr.zero, x)
+    }
+  }
+  return(1 - x)
+}
+
+#' @describeIn T2Afr Method for filter spectra
+#'
+#' @export
+#'
+T2Afr.filter_spct <- function(x,
+                              action = "add",
+                              byref = FALSE,
+                              clean = FALSE,
+                              ...) {
+  if (byref) {
+    name <- substitute(x)
+  }
+  Tfr.type <- getTfrType(x)
+  if (exists("Afr", x, inherits=FALSE)) {
+    NULL
+  } else {
+    x <- A2T(x)
+    if (clean) {
+      x <- using_Tfr(clean(x))
+    }
+    x[["Afr"]] <- 1 - x[["Tfr"]]
+    setAfrType(x, getTfrType(x))
+  }
+  if (action=="replace" && exists("Tfr", x, inherits=FALSE)) {
+    x[["Tfr"]] <- NULL
+  }
+  if (byref && is.name(name)) {  # this is a temporary safe net
+    name <- as.character(name)
+    assign(name, x, parent.frame(), inherits = TRUE)
+  }
+  return(x)
+}
+
+#' @describeIn T2Afr Method for object spectra
+#'
+#' @export
+#'
+T2Afr.object_spct <- function(x,
+                              action = "add",
+                              byref = FALSE,
+                              clean = FALSE, ...) {
+  if (byref) {
+    name <- substitute(x)
+  }
+  if (exists("Afr", x, inherits=FALSE)) {
+    NULL
+  } else {
+    if (clean) {
+      x <- using_Tfr(clean(x))
+    }
+    Tfr.type <- getTfrType(x)
+    if (Tfr.type == "total") {
+      x[["Afr"]] <- 1 - x[["Tfr"]]
+    } else if ((Tfr.type == "internal")) {
+      x[["Afr"]] <- 1 - (x[["Tfr"]] / (1 - x[["Rfr"]]))
+    } else {
+      stop("Invalid 'Tfr.type' attribute: ", Tfr.type)
+    }
+    setAfrType(x, "total")
+  }
+  if (byref && is.name(name)) {  # this is a temporary safe net
+    name <- as.character(name)
+    assign(name, x, parent.frame(), inherits = TRUE)
+  }
+  if (any((x[["Afr"]] < 0) | (x[["Afr"]] > 1))) {
+    warning("Off-boundary absorptance values generated (range = ",
+            formatted_range(x[["Afr"]]), ")!",
+            "Possible causes: fluorescence, measurement error or bad estimate of reflectance.")
+  }
+  return(x)
+}
+
+#' @describeIn T2Afr Method for collections of filter spectra
+#'
+#' @export
+#'
+T2Afr.filter_mspct <- function(x, action = "add", byref = FALSE, clean = FALSE, ...) {
+  msmsply(x, T2Afr, action = action, byref = byref, clean = FALSE, ...)
+}
+
+#' @describeIn T2Afr Method for collections of object spectra
+#'
+#' @export
+#'
+T2Afr.object_mspct <- T2Afr.filter_mspct
+
+# T2T ---------------------------------------------------------------------
+
+#' Convert transmittance type.
+#'
+#' Function that allows conversion between internal and total transmittance.
+#'
+#' @param x,y R objects
+#' @param byref logical indicating if new object will be created by reference or by copy of x
+#' @param Tfr.type.out character One of "total" or "internal"
+#' @param ... not used in current version
+#'
+#' @export T2T
+#' @family quantity conversion functions
+#'
+T2T <- function(x, y, byref, Tfr.type.out, ...) UseMethod("T2T")
+
+#' @describeIn T2T Defauly method
+#'
+#' @export
+#'
+T2T.default <- function(x, y, byref = NULL, Tfr.type.out = NULL, ...) {
+  stop("Method 'T2T()' not implemented for class: ", class(x))
+}
+
+#' @describeIn T2T Method for filter spectra
+#'
+#' @export
+#'
+T2T.filter_spct <- function(x, y,
+                            byref = FALSE,
+                            Tfr.type.out = "total",
+                            ...) {
+  stopifnot(exists("Tfr", x, inherits=FALSE))
+  Tfr.type <- getTfrType(x)
+  if (Tfr.type == Tfr.type.out) {
+    return(x)
+  }
+  stopifnot(is.reflector_spct(y) ||
+              (is.numeric(y) && ((length(y) == nrow(x)) || (length(y) != 1))))
+  if (is.reflector_spct(y)) {
+    y <- interpolate_spct(y, w.length.out = x[["w.length"]])[["Rfr"]]
+  }
+  if (byref) {
+    name <- substitute(x)
+  }
+  if (Tfr.type == "total" && Tfr.type.out == "internal") {
+    x[["Tfr"]] <- x[["Tfr"]] / (1 - y)
+    setTfrType(x, "internal")
+  } else if (Tfr.type == "internal" && Tfr.type.out == "total") {
+    x[["Tfr"]] <- x[["Tfr"]] * (1 - y)
+    setTfrType(x, "total")
+  } else if (is.na(Tfr.type)) {
+    warning("Conversion failed because of missing 'Tfr.type' attribute.")
+  }
+  setTfrType(x, Tfr.type.out)
+  if (byref && is.name(name)) {  # this is a temporary safe net
+    name <- as.character(name)
+    assign(name, x, parent.frame(), inherits = TRUE)
+  }
+  if (any((x[["Tfr"]] < 0) | (x[["Tfr"]] > 1))) {
+    warning("Off-boundary transmittance values generated (range = ",
+            formatted_range(x[["Tfr"]]), ")!",
+            "Possible causes: fluorescence, measurement error or bad estimate of reflectance.")
+  }
+  return(x)
+}
+
+#' @describeIn T2T Method for object spectra
+#'
+#' @export
+#'
+T2T.object_spct <- function(x, y = NULL,
+                            byref = FALSE,
+                            Tfr.type.out = "total",
+                            ...) {
+  Tfr.type <- getTfrType(x)
+  if (Tfr.type == Tfr.type.out) {
+    return(x)
+  }
+  stopifnot(is.null(y))
+  if (byref) {
+    name <- substitute(x)
+  }
+  if (Tfr.type == "total" && Tfr.type.out == "internal") {
+    x[["Tfr"]] <- x[["Tfr"]] / (1 - x[["Rfr"]])
+    setTfrType(x, "internal")
+  } else if (Tfr.type == "internal" && Tfr.type.out == "total") {
+    x[["Tfr"]] <- x[["Tfr"]] * (1 - x[["Rfr"]])
+    setTfrType(x, "total")
+  } else if (is.na(Tfr.type)) {
+    warning("Conversion failed because of missing 'Tfr.type' attribute.")
+  }
+  if (byref && is.name(name)) {  # this is a temporary safe net
+    name <- as.character(name)
+    assign(name, x, parent.frame(), inherits = TRUE)
+  }
+  if (any((x[["Tfr"]] < 0) | (x[["Tfr"]] > 1))) {
+    warning("Off-boundary transmittance values generated (range = ",
+            formatted_range(x[["Tfr"]]), ")!",
+            "Possible causes: fluorescence, measurement error or bad estimate of reflectance.")
+  }
+  return(x)
+}
+
+
+#' @describeIn T2T Method for collections of filter spectra
+#'
+#' @export
+#'
+T2T.filter_mspct <- function(x, y,
+                             byref = FALSE,
+                             Tfr.type.out = "total",
+                             ...) {
+  msmsply(x, T2T, y = y, byref = byref, Tfr.type.out = Tfr.type.out, ...)
+}
+
+#' @describeIn T2T Method for collections of object spectra
+#'
+#' @export
+#'
+T2T.object_mspct <- T2T.filter_mspct
 
 # energy - photon and photon - energy conversions -------------------------
 
@@ -1628,3 +1977,214 @@ q2e.response_mspct <- function(x, action = "add", byref = FALSE, ...) {
   msmsply(x, q2e, action = action, byref = byref, ...)
 }
 
+
+# using options -----------------------------------------------------------
+
+#' Use photobiology options
+#'
+#' Execute an R expression, possibly compound, using a certain setting for
+#' spectral data related options.
+#'
+#' @param expr an R expression to execute.
+#'
+#' @return The value returned by the execution of \code{expression}.
+#'
+#' @references Based on \code{withOptions()} as offered by Thomas Lumley, and
+#'   listed in\url{http://www.burns-stat.com/the-options-mechanism-in-r/},
+#'   section Deep End, of "The Options mechanism in R" by Patrick Burns.
+#'
+#' @export
+#'
+using_Tfr <- function(expr) {
+  old <- options(photobiology.filter.qty = "transmittance")
+  on.exit(options(old))
+  expr <- substitute(expr)
+  eval.parent(expr)
+}
+
+#' @rdname using_Tfr
+#'
+#' @export
+#'
+using_Afr <- function(expr) {
+  old <- options(photobiology.filter.qty = "absorptance")
+  on.exit(options(old))
+  expr <- substitute(expr)
+  eval.parent(expr)
+}
+
+#' @rdname using_Tfr
+#'
+#' @export
+#'
+using_A <- function(expr) {
+  old <- options(photobiology.filter.qty = "absorbance")
+  on.exit(options(old))
+  expr <- substitute(expr)
+  eval.parent(expr)
+}
+
+#' @rdname using_Tfr
+#'
+#' @export
+#'
+using_energy <- function(expr) {
+  old <- options(photobiology.radiation.unit = "energy")
+  on.exit(options(old))
+  expr <- substitute(expr)
+  eval.parent(expr)
+}
+
+#' @rdname using_Tfr
+#'
+#' @export
+#'
+using_photon <- function(expr) {
+  old <- options(photobiology.radiation.unit = "photon")
+  on.exit(options(old))
+  expr <- substitute(expr)
+  eval.parent(expr)
+}
+
+#' @rdname using_Tfr
+#'
+#' @export
+#'
+using_quantum <- using_photon
+
+# Set options -----------------------------------------------------------
+
+#' Set spectral-data options
+#'
+#' Set spectral-data related options easily.
+#'
+#' @return Previous value of the modified option.
+#'
+#' @export
+#'
+energy_as_default <- function() {
+  options(photobiology.radiation.unit = "energy")
+}
+
+#' @rdname energy_as_default
+#'
+#' @export
+#'
+photon_as_default <- function() {
+  options(photobiology.radiation.unit = "photon")
+}
+
+#' @rdname energy_as_default
+#'
+#' @export
+#'
+quantum_as_default <- photon_as_default
+
+#' @rdname energy_as_default
+#'
+#' @export
+#'
+Tfr_as_default <- function() {
+  options(photobiology.filter.qty = "transmittance")
+}
+
+#' @rdname energy_as_default
+#'
+#' @export
+#'
+Afr_as_default <- function() {
+  options(photobiology.filter.qty = "absorptance")
+}
+
+#' @rdname energy_as_default
+#'
+#' @export
+#'
+A_as_default <- function() {
+  options(photobiology.filter.qty = "absorbance")
+}
+
+#' Set computation options
+#'
+#' Set computation related options easily.
+#'
+#' @param flag logical.
+#'
+#' @return Previous value of the modified option.
+#'
+#' @export
+#'
+wb_trim_as_default <- function(flag = TRUE) {
+  options(photobiology.waveband.trim = flag)
+}
+
+#' @rdname wb_trim_as_default
+#'
+#' @export
+#'
+use_cached_mult_as_default <- function(flag = TRUE) {
+  options(photobiology.use.cached.mult = flag)
+}
+
+#' @rdname energy_as_default
+#'
+#' @export
+#'
+unset_user_defaults <- function() {
+  options(photobiology.filter.qty = NULL,
+          photobiology.radiation.unit = NULL,
+          photobiology.verbose = NULL,
+          photobiology.strict.range = NULL,
+          photobiology.waveband.trim = NULL,
+          photobiology.use.cached.mult = NULL)
+}
+
+#' @rdname energy_as_default
+#'
+#' @export
+#'
+unset_radiation_unit_default <- function() {
+  options(photobiology.radiation.unit = NULL)
+}
+
+#' @rdname energy_as_default
+#'
+#' @export
+#'
+unset_filter_qty_default <- function() {
+  options(photobiology.filter.qty = NULL)
+}
+
+#' @rdname energy_as_default
+#'
+#' @export
+#'
+unset_user_defaults <- function() {
+  options(photobiology.filter.qty = NULL,
+          photobiology.radiation.unit = NULL,
+          photobiology.verbose = NULL,
+          photobiology.strict.range = NULL,
+          photobiology.waveband.trim = NULL,
+          photobiology.use.cached.mult = NULL)
+}
+#' Set error reporting options
+#'
+#' Set error reporting related options easily.
+#'
+#' @param flag logical.
+#'
+#' @return Previous value of the modified option.
+#'
+#' @export
+#'
+verbose_as_default <- function(flag = TRUE) {
+  options(photobiology.verbose = flag)
+}
+
+#' @rdname verbose_as_default
+#'
+#' @export
+#'
+strict_range_as_default <- function(flag = TRUE) {
+  options(photobiology.strict.range = flag)
+}
