@@ -320,7 +320,8 @@ is.any_mspct <- function(x) {
 #'
 #' @export
 #'
-#' @family creation of spectral objects functions
+#' @family conversion of collections of spectra
+#'
 #' @rdname as.generic_mspct
 #'
 as.generic_mspct <- function(x, force.spct.class = FALSE) {
@@ -466,7 +467,7 @@ as.chroma_mspct <- function(x) {
 #'
 #' @export
 #'
-#' @family collections of spectra classes family
+#' @family conversion of collections of spectra
 #'
 split2mspct <- function(x,
                         member.class = NULL,
@@ -479,6 +480,9 @@ split2mspct <- function(x,
     stop("'split2mspct()' is for slicing vertically wide data in data frames ",
          "'subset2mspct()' is used in the case of tidy data in long form.")
   }
+  if (!is.numeric(x[[w.length.var]])) {
+    stop("Non-numeric variable '", w.length.var, "' is bad for wavelengths.")
+  }
   collection.class <- sub("_spct", "_mspct", member.class, fixed = TRUE)
   member.constr <- member.class
   collection.constr <- collection.class
@@ -487,6 +491,7 @@ split2mspct <- function(x,
   l <- list()
   for (col in data.cols) {
     if (!is.numeric(x[[col]])) {
+      warning("Skipping non-numeric column in x: ", col)
       next
     }
     args <- list(w.length = x[[w.length.var]])
@@ -625,7 +630,7 @@ split2raw_mspct <- function(x,
 #'
 #' @export
 #'
-#' @family collections of spectra classes family
+#' @family conversion of collections of spectra
 #'
 subset2mspct <- function(x,
                          member.class = NULL,
